searchNodes=[{"doc":"This module defines the behaviour for implementing clustering strategies.","ref":"Cluster.Strategy.html","title":"Cluster.Strategy","type":"behaviour"},{"doc":"","ref":"Cluster.Strategy.html#c:child_spec/1","title":"Cluster.Strategy.child_spec/1","type":"callback"},{"doc":"Given a list of node names, attempts to connect to all of them. Returns :ok if all nodes connected, or {:error, [{node, reason}, ..]} if we failed to connect to some nodes. All failures are logged.","ref":"Cluster.Strategy.html#connect_nodes/4","title":"Cluster.Strategy.connect_nodes/4","type":"function"},{"doc":"","ref":"Cluster.Strategy.html#difference/2","title":"Cluster.Strategy.difference/2","type":"function"},{"doc":"Given a list of node names, attempts to disconnect from all of them. Returns :ok if all nodes disconnected, or {:error, [{node, reason}, ..]} if we failed to disconnect from some nodes. All failures are logged.","ref":"Cluster.Strategy.html#disconnect_nodes/4","title":"Cluster.Strategy.disconnect_nodes/4","type":"function"},{"doc":"","ref":"Cluster.Strategy.html#intersection/2","title":"Cluster.Strategy.intersection/2","type":"function"},{"doc":"","ref":"Cluster.Strategy.html#c:start_link/1","title":"Cluster.Strategy.start_link/1","type":"callback"},{"doc":"","ref":"Cluster.Strategy.html#t:bad_nodes/0","title":"Cluster.Strategy.bad_nodes/0","type":"type"},{"doc":"","ref":"Cluster.Strategy.html#t:mfa_tuple/0","title":"Cluster.Strategy.mfa_tuple/0","type":"type"},{"doc":"","ref":"Cluster.Strategy.html#t:strategy_args/0","title":"Cluster.Strategy.strategy_args/0","type":"type"},{"doc":"","ref":"Cluster.Strategy.html#t:topology/0","title":"Cluster.Strategy.topology/0","type":"type"},{"doc":"Assumes you have nodes that respond to the specified DNS query (A record), and which follow the node name pattern of &lt;name&gt;@&lt;ip-address&gt; . If your setup matches those assumptions, this strategy will periodically poll DNS and connect all nodes it finds. Options poll_interval - How often to poll in milliseconds (optional; default: 5_000) query - DNS query to use (required; e.g. &quot;my-app.example.com&quot;) node_basename - The short name of the nodes you wish to connect to (required; e.g. &quot;my-app&quot;) Usage config :libcluster , topologies : [ dns_poll_example : [ strategy : Elixir.Cluster.Strategy.DNSPoll , config : [ polling_interval : 5_000 , query : &quot;my-app.example.com&quot; , node_basename : &quot;my-app&quot; ] ] ]","ref":"Cluster.Strategy.DNSPoll.html","title":"Cluster.Strategy.DNSPoll","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.DNSPoll.html#child_spec/1","title":"Cluster.Strategy.DNSPoll.child_spec/1","type":"function"},{"doc":"","ref":"Cluster.Strategy.DNSPoll.html#lookup_all_ips/1","title":"Cluster.Strategy.DNSPoll.lookup_all_ips/1","type":"function"},{"doc":"","ref":"Cluster.Strategy.DNSPoll.html#start_link/1","title":"Cluster.Strategy.DNSPoll.start_link/1","type":"function"},{"doc":"This clustering strategy relies on Erlang's built-in distribution protocol. You can have libcluster automatically connect nodes on startup for you by configuring the strategy like below: config :libcluster , topologies : [ epmd_example : [ strategy : Elixir.Cluster.Strategy.Epmd , config : [ timeout : 30_000 , hosts : [ :&quot;a@127.0.0.1&quot; , :&quot;b@127.0.0.1&quot; ] ] ] ] An optional timeout can be specified in the config. This is the timeout that will be used in the GenServer to connect the nodes. This defaults to :infinity meaning that the connection process will only happen when the worker is started. Any integer timeout will result in the connection process being triggered. In the example above, it has been configured for 30 seconds.","ref":"Cluster.Strategy.Epmd.html","title":"Cluster.Strategy.Epmd","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Epmd.html#child_spec/1","title":"Cluster.Strategy.Epmd.child_spec/1","type":"function"},{"doc":"This clustering strategy relies on Erlang's built-in distribution protocol by using a .hosts.erlang file (as used by the :net_adm module). Please see the net_adm docs for more details. In short, the following is the gist of how it works: File .hosts.erlang consists of a number of host names written as Erlang terms. It is looked for in the current work directory, the user's home directory, and $OTP_ROOT (the root directory of Erlang/OTP), in that order. This looks a bit like the following in practice: &#39;super.eua.ericsson.se&#39; . &#39;renat.eua.ericsson.se&#39; . &#39;grouse.eua.ericsson.se&#39; . &#39;gauffin1.eua.ericsson.se&#39; . You can have libcluster automatically connect nodes on startup for you by configuring the strategy like below: config :libcluster , topologies : [ erlang_hosts_example : [ strategy : Elixir.Cluster.Strategy.ErlangHosts , config : [ timeout : 30_000 ] ] ] An optional timeout can be specified in the config. This is the timeout that will be used in the GenServer to connect the nodes. This defaults to :infinity meaning that the connection process will only happen when the worker is started. Any integer timeout will result in the connection process being triggered. In the example above, it has been configured for 30 seconds.","ref":"Cluster.Strategy.ErlangHosts.html","title":"Cluster.Strategy.ErlangHosts","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.ErlangHosts.html#child_spec/1","title":"Cluster.Strategy.ErlangHosts.child_spec/1","type":"function"},{"doc":"Callback implementation for Cluster.Strategy.start_link/1 .","ref":"Cluster.Strategy.ErlangHosts.html#start_link/1","title":"Cluster.Strategy.ErlangHosts.start_link/1","type":"function"},{"doc":"This clustering strategy uses multicast UDP to gossip node names to other nodes on the network. These packets are listened for on each node as well, and a connection will be established between the two nodes if they are reachable on the network, and share the same magic cookie. In this way, a cluster of nodes may be formed dynamically. The gossip protocol is extremely simple, with a prelude followed by the node name which sent the packet. The node name is parsed from the packet, and a connection attempt is made. It will fail if the two nodes do not share a cookie. By default, the gossip occurs on port 45892, using the multicast address 233.252.1.32 The gossip protocol is not encrypted by default, but can be by providing a secret in the configuration of the strategy (as shown below). This can also be used to run multiple clusters with the same multicast configuration, as nodes not sharing the same encryption key will not be connected. You may configure the multicast interface, multicast address, the interface address to bind to, the port, the TTL of the packets and the optional secret using the following settings: config :libcluster , topologies : [ gossip_example : [ strategy : Elixir.Cluster.Strategy.Gossip , config : [ port : 45892 , if_addr : &quot;0.0.0.0&quot; , multicast_if : &quot;192.168.1.1&quot; , multicast_addr : &quot;233.252.1.32&quot; , multicast_ttl : 1 , secret : &quot;somepassword&quot; ] ] ] A TTL of 1 will limit packets to the local network, and is the default TTL. Optionally, broadcast_only: true option can be set which disables multicast and only uses broadcasting. This limits connectivity to local network but works on in scenarios where multicast is not enabled. Use multicast_addr as the broadcast address. Example for broadcast only: config :libcluster , topologies : [ gossip_example : [ strategy : Elixir.Cluster.Strategy.Gossip , config : [ port : 45892 , if_addr : &quot;0.0.0.0&quot; , multicast_addr : &quot;255.255.255.255&quot; , broadcast_only : true ] ] ] Debug logging is deactivated by default for this clustering strategy, but it can be easily activated by configuring the application: use Mix.Config config :libcluster , debug : true All the checks are done at runtime, so you can flip the debug level without being forced to shutdown your node.","ref":"Cluster.Strategy.Gossip.html","title":"Cluster.Strategy.Gossip","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Gossip.html#child_spec/1","title":"Cluster.Strategy.Gossip.child_spec/1","type":"function"},{"doc":"Callback implementation for Cluster.Strategy.start_link/1 .","ref":"Cluster.Strategy.Gossip.html#start_link/1","title":"Cluster.Strategy.Gossip.start_link/1","type":"function"},{"doc":"This clustering strategy works by fetching information of endpoints or pods, which are filtered by given Kubernetes namespace and label. This strategy requires a service account with the ability to list endpoints or pods. If you want to avoid that, you could use one of the DNS-based strategies instead. See Cluster.Strategy.Kubernetes.DNS and Cluster.Strategy.Kubernetes.DNSSRV . It assumes that all Erlang nodes are using longnames - &lt;basename&gt;@&lt;ip_or_domain&gt; : all nodes are using the same &lt;basename&gt; all nodes are using unique &lt;ip_or_domain&gt; In &lt;basename&gt;@&lt;ip_or_domain&gt; : &lt;basename&gt; would be the value configured by :kubernetes_node_basename option. &lt;ip_or_domain&gt; would be the value which is controlled by following options: :kubernetes_namespace :kubernetes_selector :kubernetes_service_name :kubernetes_ip_lookup_mode :mode Getting &lt;basename&gt; As said above, the basename is configured by :kubernetes_node_basename option. Just one thing to keep in mind - when building an OTP release, make sure that the name of the OTP release matches the name configured by :kubernetes_node_basename . Getting &lt;ip_or_domain&gt; :kubernetes_namespace and :kubernetes_selector option These two options configure how to filter required endpoints or pods. :kubernetes_ip_lookup_mode option These option configures where to lookup the required IP. Available values: :endpoints (default) :pods :endpoints When setting this value, this strategy will lookup IP from endpoints. In order for your endpoints to be found they should be returned when you run: kubectl get endpoints - l app = myapp Then, this strategy will fetch the addresses of all endpoints with that label and attempt to connect. :pods When setting this value, this strategy will lookup IP from pods directly. In order for your pods to be found they should be returned when you run: kubectl get pods - l app = myapp Then, this strategy will fetch the IP of all pods with that label and attempt to connect. :mode option These option configures how to build the longname. Available values: :ip (default) :dns :hostname :ip In this mode, the IP address is used directly. The longname will be something like: myapp @ &lt; ip &gt; Getting this mode to work requires: exposing pod IP from Kubernetes to the Erlang node. setting the name of Erlang node according to the exposed information First, expose required information from Kubernetes as environment variables of Erlang node: # deployment.yaml env : - name : POD_IP valueFrom : fieldRef : fieldPath : status . podIP Then, set the name of Erlang node by using the exposed environment variables. If you use mix releases, you can configure the required options in rel/env.sh.eex : # rel/env.sh.eex export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &lt; % = @release . name % &gt; @ $ { POD_IP } export RELEASE_DISTRIBUTION=name will append a -name option to the start command directly and requires no further changes to the vm.args . :hostname In this mode, the hostname is used directly. The longname will be something like: myapp @ &lt; hostname &gt; . &lt; service_name &gt; . &lt; namespace &gt; . svc . &lt; cluster_domain &gt; Getting :hostname mode to work requires: deploying pods as a StatefulSet (otherwise, hostname is not set for pods) setting :kubernetes_service_name to the name of the Kubernetes service that is being lookup setting the name of Erlang node according to hostname of pods Then, set the name of Erlang node by using the hostname of pod. If you use mix releases, you can configure the required options in rel/env.sh.eex : # rel/env.sh.eex export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &lt; % = @release . name % &gt; @ $ ( hostname - f ) hostname -f returns the whole FQDN, which is something like: $(hostname).${SERVICE_NAME}.${NAMESPACE}.svc.${CLUSTER_DOMAIN}&quot; . :dns In this mode, an IP-based pod A record is used. The longname will be something like: myapp @ &lt; pod_a_record &gt; . &lt; namespace &gt; . pod . &lt; cluster_domain &gt; Getting :dns mode to work requires: exposing pod IP from Kubernetes to the Erlang node setting the name of Erlang node according to the exposed information First, expose required information from Kubernetes as environment variables of Erlang node: # deployment.yaml env : - name : NAMESPACE valueFrom : fieldRef : fieldPath : metadata . namespace - name : POD_IP valueFrom : fieldRef : fieldPath : status . podIP Then, set the name of Erlang node by using the exposed environment variables. If you use mix releases, you can configure the required options in rel/env.sh.eex : # rel/env.sh.eex export POD_A_RECORD = $ ( echo $ POD_IP | sed &#39;s/./-/g&#39; ) export CLUSTER_DOMAIN = cluster . local # modify this value according to your actual situation export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &lt; % = @release . name % &gt; @ $ { POD_A_RECORD } . $ { NAMESPACE } . pod . $ { CLUSTER_DOMAIN } Which mode is the best one? There is no best, only the best for you: If you're not using a StatefulSet, use :ip or :dns . If you're using a StatefulSet, use :hostname . And, there is one thing that can be taken into consideration. When using :ip or :dns , you can establish a remote shell (as well as run observer) by using kubectl port-forward in combination with some entries in /etc/hosts . Polling Interval The default interval to sync topologies is 5000 (5 seconds). You can configure it with :polling_interval option. Getting cluster information In general, you don't need to read this, the default values will work. This strategy fetchs information of endpoints or pods by accessing the REST API provided by Kubernetes. The base URL of the REST API has two parts: &lt; master_name &gt; . &lt; cluster_domain &gt; &lt;master_name&gt; is configured by following options: :kubernetes_master - the default value is kubernetes.default.svc &lt;cluster_domain&gt; is configured by following options and environment variables: :kubernetes_cluster_name - the default value is cluster , and the final cluster domain will be &lt;cluster_name&gt;.local CLUSTER_DOMAIN - when this environment variable is provided, :kubernetes_cluster_name will be ignored &lt;master_name&gt; and &lt;cluster_domain&gt; also affect each other, checkout the source code for more details. Besides the base URL of the REST API, a service account must be provided. The service account is configured by following options: :kubernetes_service_account_path - the default value is /var/run/secrets/kubernetes.io/serviceaccount An example configuration config :libcluster , topologies : [ erlang_nodes_in_k8s : [ strategy : Elixir.Cluster.Strategy.Kubernetes , config : [ mode : :ip , kubernetes_node_basename : &quot;myapp&quot; , kubernetes_selector : &quot;app=myapp&quot; , kubernetes_namespace : &quot;my_namespace&quot; , polling_interval : 10_000 ] ] ]","ref":"Cluster.Strategy.Kubernetes.html","title":"Cluster.Strategy.Kubernetes","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Kubernetes.html#child_spec/1","title":"Cluster.Strategy.Kubernetes.child_spec/1","type":"function"},{"doc":"Callback implementation for Cluster.Strategy.start_link/1 .","ref":"Cluster.Strategy.Kubernetes.html#start_link/1","title":"Cluster.Strategy.Kubernetes.start_link/1","type":"function"},{"doc":"This clustering strategy works by fetching IP addresses with the help of a headless service in current Kubernetes namespace. This strategy requires exposing pods by a headless service. If you want to avoid that, you could use Cluster.Strategy.Kubernetes . It assumes that all Erlang nodes are using longnames - &lt;basename&gt;@&lt;ip&gt; : all nodes are using the same &lt;basename&gt; all nodes are using unique &lt;ip&gt; In &lt;basename&gt;@&lt;ip&gt; : &lt;basename&gt; would be the value configured by :application_name option. &lt;ip&gt; would be the value which is controlled by following options: :service :resolver Getting &lt;basename&gt; As said above, the basename is configured by :application_name option. Just one thing to keep in mind - when building an OTP release, make sure that the name of the OTP release matches the name configured by :application_name . Getting &lt;ip&gt; It will fetch IP addresses of all pods under a headless service and attempt to connect. Setup Getting this strategy to work requires: exposing pod IP from Kubernetes to the Erlang node. setting a headless service for the pods setting the name of Erlang node according to the exposed information First, expose required information from Kubernetes as environment variables of Erlang node: # deployment.yaml env : - name : POD_IP valueFrom : fieldRef : fieldPath : status . podIP Second, set a headless service for the pods: # deployment.yaml apiVersion : v1 kind : Service metadata : name : myapp - headless spec : selector : app : myapp type : ClusterIP clusterIP : None Then, set the name of Erlang node by using the exposed environment variables. If you use mix releases, you can configure the required options in rel/env.sh.eex : # rel/env.sh.eex export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &lt; % = @release . name % &gt; @ $ { POD_IP } Polling Interval The default interval to sync topologies is 5000 (5 seconds). You can configure it with :polling_interval option. An example configuration config :libcluster , topologies : [ erlang_nodes_in_k8s : [ strategy : Elixir.Cluster.Strategy.Kubernetes.DNS , config : [ service : &quot;myapp-headless&quot; , application_name : &quot;myapp&quot; , polling_interval : 10_000 ] ] ]","ref":"Cluster.Strategy.Kubernetes.DNS.html","title":"Cluster.Strategy.Kubernetes.DNS","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Kubernetes.DNS.html#child_spec/1","title":"Cluster.Strategy.Kubernetes.DNS.child_spec/1","type":"function"},{"doc":"This clustering strategy works by issuing a SRV query for the headless service where the StatefulSet containing your nodes is running. This strategy requires deploying pods as a StatefulSet which is exposed by a headless service. If you want to avoid that, you could use Cluster.Strategy.Kubernetes.DNS . It assumes that all Erlang nodes are using longnames - &lt;basename&gt;@&lt;domain&gt; : all nodes are using the same &lt;basename&gt; all nodes are using unique &lt;domain&gt; In &lt;basename&gt;@&lt;domain&gt; : &lt;basename&gt; would be the value configured by :application_name option. &lt;domain&gt; would be the value which is controlled by following options: :service :namespace :resolver Getting &lt;basename&gt; As said above, the basename is configured by :application_name option. Just one thing to keep in mind - when building an OTP release, make sure that the name of the OTP release matches the name configured by :application_name . Getting &lt;domain&gt; For more information, see the kubernetes stateful-application documentation Setup Getting this strategy to work requires: deploying pods as a StatefulSet (otherwise, hostname won't set for pods) exposing above StatefulSet by a headless service (otherwise, the SRV query won't work as expected) setting the name of Erlang node according to hostname of pods First, deploying pods as a StatefulSet which is exposed by a headless service. And here is an example of a corresponding Kubernetes definition: apiVersion: v1 kind: Service metadata: name: &quot;myapp-headless&quot; labels: app: myapp spec: ports: - port: 4000 name: web clusterIP: None selector: app: myapp --- apiVersion: apps/v1 kind: StatefulSet metadata: name: myapp spec: serviceName: &quot;myapp-headless&quot; replicas: 2 selector: matchLabels: app: myapp template: metadata: labels: app: myapp spec: containers: - name: myapp image: myapp:v1.0.0 imagePullPolicy: Always ports: - containerPort: 4000 name: http protocol: TCP Then, set the name of Erlang node by using the hostname of pod. If you use mix releases, you can configure the required options in rel/env.sh.eex : # rel/env.sh.eex export RELEASE_DISTRIBUTION = name export RELEASE_NODE = &lt; % = @release . name % &gt; @ $ ( hostname - f ) Polling Interval The default interval to sync topologies is 5000 (5 seconds). You can configure it with :polling_interval option. An example configuration config :libcluster , topologies : [ erlang_nodes_in_k8s : [ strategy : Elixir.Cluster.Strategy.Kubernetes.DNSSRV , config : [ service : &quot;myapp-headless&quot; , application_name : &quot;myapp&quot; , namespace : &quot;default&quot; , polling_interval : 10_000 ] ] ] An example of how this strategy extracts topology information from DNS $ hostname -f myapp-1.myapp-headless.default.svc.cluster.local # An SRV query for a headless service returns multiple entries $ dig SRV myapp-headless.default.svc.cluster.local ; &lt;&lt;&gt;&gt; DiG 9.14.3 &lt;&lt;&gt;&gt; SRV myapp-headless.default.svc.cluster.local ;; global options: +cmd ;; Got answer: ;; WARNING: .local is reserved for Multicast DNS ;; You are currently testing what happens when an mDNS query is leaked to DNS ;; -&gt;&gt;HEADER&lt;&lt;- opcode: QUERY, status: NOERROR, id: 7169 ;; flags: qr aa rd ra; QUERY: 1, ANSWER: 2, AUTHORITY: 0, ADDITIONAL: 2 ;; QUESTION SECTION: ;myapp-headless.default.svc.cluster.local. IN SRV ;; ANSWER SECTION: myapp-headless.default.svc.cluster.local. 30 IN SRV 10 50 0 myapp-0.myapp-headless.default.svc.cluster.local. myapp-headless.default.svc.cluster.local. 30 IN SRV 10 50 0 myapp-1.myapp-headless.default.svc.cluster.local. ;; ADDITIONAL SECTION: myapp-0.myapp-headless.default.svc.cluster.local. 30 IN A 10.1.0.95 myapp--1.myapp-headless.default.svc.cluster.local. 30 IN A 10.1.0.96 ;; Query time: 0 msec ;; SERVER: 10.96.0.10#53(10.96.0.10) ;; WHEN: Wed Jul 03 11:55:27 UTC 2019 ;; MSG SIZE rcvd: 167","ref":"Cluster.Strategy.Kubernetes.DNSSRV.html","title":"Cluster.Strategy.Kubernetes.DNSSRV","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Kubernetes.DNSSRV.html#child_spec/1","title":"Cluster.Strategy.Kubernetes.DNSSRV.child_spec/1","type":"function"},{"doc":"This clustering strategy relies on Erlang's built-in distribution protocol. Unlike Cluster.Strategy.Empd, this strategy assumes that all nodes are on the local host and can be discovered by epmd. Make sure epmd is started before you start your application, or startup will fail. When running with mix , you can do this automatically by passing the --name or --sname flag to start distribution. It should be configured as follows: config :libcluster , topologies : [ local_epmd_example : [ strategy : Elixir.Cluster.Strategy.LocalEpmd ] ]","ref":"Cluster.Strategy.LocalEpmd.html","title":"Cluster.Strategy.LocalEpmd","type":"module"},{"doc":"Callback implementation for Cluster.Strategy.start_link/1 .","ref":"Cluster.Strategy.LocalEpmd.html#start_link/1","title":"Cluster.Strategy.LocalEpmd.start_link/1","type":"function"},{"doc":"This clustering strategy is specific to the Rancher container platform. It works by querying the platform's metadata API for containers belonging to the same service as the node and attempts to connect them. (see: http://rancher.com/docs/rancher/latest/en/rancher-services/metadata-service/ ) It assumes that all nodes share a base name and are using longnames of the form &lt;basename@&lt;ip&gt; where the &lt;ip&gt; is unique for each node. A way to assign a name to a node on boot in an app running as a Distillery release is: Create a wrapper script which will interpolate the current ip of the container. #!/bin/sh export CONTAINER_IP=&quot;$(hostname -I | cut -f1 -d&#39; &#39;)&quot; export REPLACE_OS_VARS=true /app/bin/app &quot;$@&quot; # vm.args - name app @ $ { CONTAINER_IP } An example configuration is below: config :libcluster , topologies : [ rancher_example : [ strategy : Elixir.Cluster.Strategy.Rancher , config : [ node_basename : &quot;myapp&quot; , polling_interval : 10_000 ] ] ]","ref":"Cluster.Strategy.Rancher.html","title":"Cluster.Strategy.Rancher","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Strategy.Rancher.html#child_spec/1","title":"Cluster.Strategy.Rancher.child_spec/1","type":"function"},{"doc":"Callback implementation for Cluster.Strategy.start_link/1 .","ref":"Cluster.Strategy.Rancher.html#start_link/1","title":"Cluster.Strategy.Rancher.start_link/1","type":"function"},{"doc":"The state of one strategy.","ref":"Cluster.Strategy.State.html","title":"Cluster.Strategy.State","type":"module"},{"doc":"","ref":"Cluster.Strategy.State.html#t:t/0","title":"Cluster.Strategy.State.t/0","type":"type"},{"doc":"This module handles supervising the configured topologies, and is designed to support being started within your own supervision tree, as shown below: defmodule MyApp.App do use Application def start ( _type , _args ) do topologies = [ example : [ strategy : Cluster.Strategy.Epmd , config : [ hosts : [ :&quot;a@127.0.0.1&quot; , :&quot;b@127.0.0.1&quot; ] ] , ] ] children = [ { Cluster.Supervisor , [ topologies , [ name : MyApp.ClusterSupervisor ] ] } , .. other children .. ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor ) end end The topologies configuration structure shown above can be built manually, like shown, so that you can load config at runtime in a way that best suits your application; or if you don't need to do any special config handling, you can use the Mix config file, and just use Application.get_env(:libcluster, :topologies) . That config would look like so: config :libcluster , topologies : [ example : [ ... ] ] Use the method most convenient for you.","ref":"Cluster.Supervisor.html","title":"Cluster.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cluster.Supervisor.html#child_spec/1","title":"Cluster.Supervisor.child_spec/1","type":"function"},{"doc":"Start a new instance of this supervisor. This is the callback indicated in the child specification returned by child_spec/1 . It expects a list of the form [config, supervisor_opts] , or [config] . The former allows you to provide options for the supervisor like with Supervisor.start_link/3 .","ref":"Cluster.Supervisor.html#start_link/1","title":"Cluster.Supervisor.start_link/1","type":"function"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Use new cypher names Allow Epmd strategy to reconnect after connection failures Detect Self Signed Certificate Authority for Kubernetes Strategy Remove calls to deprecated Logger.warn/2 3.3.0 Changed Default multicast address is now 233.252.1.32, was 230.1.1.251, commit 2.3.0 Added Clustering strategy for the Rancher container platform (see: https://github.com/rancher/rancher ) LocalEpmd strategy that uses epmd to discover nodes on the local host Gossip strategy multicast interface is used for adding multicast membership","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Added Configurable connect and disconnect options for implementing strategies on top of custom topologies The ability to start libcluster for more than a single topology Added polling_interval option to Kubernetes strategy Added ability to specify a list of hosts for the Epmd strategy to connect to on start Removed Cluster.Events module, as it was redundant and unused Changed Configuration format has changed significantly, please review the docs","ref":"changelog.html#2-0-0","title":"Changelog - 2.0.0","type":"extras"},{"doc":"This library provides a mechanism for automatically forming clusters of Erlang nodes, with either static or dynamic node membership. It provides a pluggable &quot;strategy&quot; system, with a variety of strategies provided out of the box. You can find supporting documentation here .","ref":"readme.html","title":"libcluster","type":"extras"},{"doc":"Automatic cluster formation/healing Choice of multiple clustering strategies out of the box: Standard Distributed Erlang facilities (e.g. epmd , .hosts.erlang ), which supports IP-based or DNS-based names Multicast UDP gossip, using a configurable port/multicast address, Kubernetes via its metadata API using via a configurable label selector and node basename; or alternatively, using DNS. Rancher, via its metadata API Easy to provide your own custom clustering strategies for your specific environment. Easy to use provide your own distribution plumbing (i.e. something other than Distributed Erlang), by implementing a small set of callbacks. This allows libcluster to support projects like Partisan .","ref":"readme.html#features","title":"libcluster - Features","type":"extras"},{"doc":"defp deps do [ { :libcluster , &quot;~&gt; MAJ.MIN&quot; } ] end You can determine the latest version by running mix hex.info libcluster in your shell, or by going to the libcluster page on Hex.pm .","ref":"readme.html#installation","title":"libcluster - Installation","type":"extras"},{"doc":"It is easy to get started using libcluster , simply decide which strategy you want to use to form a cluster, define a topology, and then start the Cluster.Supervisor module in the supervision tree of an application in your Elixir system, as demonstrated below: defmodule MyApp.App do use Application def start ( _type , _args ) do topologies = [ example : [ strategy : Cluster.Strategy.Epmd , config : [ hosts : [ :&quot;a@127.0.0.1&quot; , :&quot;b@127.0.0.1&quot; ] ] , ] ] children = [ { Cluster.Supervisor , [ topologies , [ name : MyApp.ClusterSupervisor ] ] } , # ..other children.. ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor ) end end The following section describes topology configuration in more detail.","ref":"readme.html#usage","title":"libcluster - Usage","type":"extras"},{"doc":"You can configure libcluster either in your Mix config file ( config.exs ) as shown below, or construct the keyword list structure manually, as shown in the previous section. Either way, you need to pass the configuration to the Cluster.Supervisor module in it's start arguments. If you prefer to use Mix config files, then simply use Application.get_env(:libcluster, :topologies) to get the config that Cluster.Supervisor expects. config :libcluster , topologies : [ epmd_example : [ # The selected clustering strategy. Required. strategy : Cluster.Strategy.Epmd , # Configuration for the provided strategy. Optional. config : [ hosts : [ :&quot;a@127.0.0.1&quot; , :&quot;b@127.0.0.1&quot; ] ] , # The function to use for connecting nodes. The node # name will be appended to the argument list. Optional connect : { :net_kernel , :connect_node , [ ] } , # The function to use for disconnecting nodes. The node # name will be appended to the argument list. Optional disconnect : { :erlang , :disconnect_node , [ ] } , # The function to use for listing nodes. # This function must return a list of node names. Optional list_nodes : { :erlang , :nodes , [ :connected ] } , ] , # more topologies can be added ... gossip_example : [ # ... ] ]","ref":"readme.html#example-configuration","title":"libcluster - Example Configuration","type":"extras"},{"doc":"For instructions on configuring each strategy included with libcluster , please visit the docs on HexDocs , and look at the module doc for the strategy you want to use. The authoritative documentation for each strategy is kept up to date with the module implementing it.","ref":"readme.html#strategy-configuration","title":"libcluster - Strategy Configuration","type":"extras"},{"doc":"You have a handful of choices with regards to cluster management out of the box: Cluster.Strategy.Epmd , which relies on epmd to connect to a configured set of hosts. Cluster.Strategy.LocalEpmd , which relies on epmd to connect to discovered nodes on the local host. Cluster.Strategy.ErlangHosts , which uses the .hosts.erlang file to determine which hosts to connect to. Cluster.Strategy.Gossip , which uses multicast UDP to form a cluster between nodes gossiping a heartbeat. Cluster.Strategy.Kubernetes , which uses the Kubernetes Metadata API to query nodes based on a label selector and basename. Cluster.Strategy.Kubernetes.DNS , which uses DNS to join nodes under a shared headless service in a given namespace. Cluster.Strategy.Rancher , which like the Kubernetes strategy, uses a metadata API to query nodes to cluster with. You can also define your own strategy implementation, by implementing the Cluster.Strategy behavior. This behavior expects you to implement a start_link/1 callback, optionally overriding child_spec/1 if needed. You don't necessarily have to start a process as part of your strategy, but since it's very likely you will need to maintain some state, designing your strategy as an OTP process (e.g. GenServer ) is the ideal method, however any valid OTP process will work. See the Cluster.Strategy module for details on the callbacks you need to implement and the arguments they receive. If you do not wish to use the default Erlang distribution protocol, you may provide an alternative means of connecting/ disconnecting nodes via the connect and disconnect configuration options, if not using Erlang distribution you must provide a list_nodes implementation as well. They take a {module, fun, args} tuple, and append the node name being targeted to the args list. How to implement distribution in this way is left as an exercise for the reader, but I recommend taking a look at the Firenest project currently under development. By default, libcluster uses Distributed Erlang. Third-Party Strategies The following list of third-party strategy implementations is not comprehensive, but are known to exist. libcluster_ec2 - EC2 clustering strategy based on tags libcluster_droplet - Digital Ocean Droplet clustering strategy libcluster_consul - Consul clustering strategy","ref":"readme.html#clustering","title":"libcluster - Clustering","type":"extras"},{"doc":"Copyright (c) 2016 Paul Schoenfelder This library is MIT licensed. See the LICENSE.md for details.","ref":"readme.html#copyright-and-license","title":"libcluster - Copyright and License","type":"extras"}]